#ifndef __DUNE_MMDG_MMDGPROBLEM2_HH__
#define __DUNE_MMDG_MMDGPROBLEM2_HH__

#include <cmath>
#include <dune/mmdg/problems/coupleddgproblem.hh>

//a coupled dg problem for dim = 2,
//taken from [Antonietti et al. (2019): Example 6.3]
template<class Coordinate, class Scalar = double>
class MMDGProblem2 : public CoupledDGProblem<Coordinate, Scalar>
{
public:
  static constexpr int dim = Coordinate::dimension;
  using Base = CoupledDGProblem<Coordinate, Scalar>;
  using Matrix = typename Base::Matrix;

  //constructor
  MMDGProblem2 (const Scalar d) : d_(d) {}

  //the exact bulk solution at position pos
  Scalar exactSolution (const Coordinate& pos) const
  {
    Scalar solution =
      (pos[0] < 0.5) ? std::sin(4 * pos[0]) : std::cos(4 * pos[0]);
    return solution * std::cos(M_PI * pos[1]);
  }

  //the exact solution on the interface at position pos
  Scalar exactInterfaceSolution (const Coordinate& pos) const
  {
    constexpr Scalar sourcefactor = 0.75 * (std::cos(2.0) + std::sin(2.0));
    return sourcefactor * std::cos(M_PI * pos[1]);
  }

  //indicates whether an exact solution is implemented for the problem
  bool hasExactSolution () const
  {
    return true;
  };

  //bulk source term at position pos
  Scalar q (const Coordinate& pos) const
  {
    constexpr Scalar sourcefactor = 16.0 + M_PI * M_PI;
    Scalar source =
      (pos[0] < 0.5) ? std::sin(4 * pos[0]) : std::cos(4 * pos[0]);
    return sourcefactor * source * std::cos(M_PI * pos[1]);
  }

  //interface source term at position pos
  Scalar qInterface (const Coordinate& pos) const
  {
    constexpr Scalar sourcefactor = std::cos(2.0) + std::sin(2.0);
    return sourcefactor * (4.0 + 0.75 * d_ * d_ * M_PI * M_PI)
      * std::cos(M_PI * pos[1]);
  }

  //aperture d of the fracture at position pos
  Scalar aperture (const Coordinate& pos) const
  {
    return d_;
  }

  //permeability per aperture  of the fracture per aperture in normal direction
  //at position pos
  Scalar Kperp (const Coordinate& pos) const
  {
    return Scalar(2.0);
  }

  //returns the recommended quadrature order to compute an integral
  //over x * q(x)
  int quadratureOrder_q () const
  {
    return 10;
  }

  //returns the recommended quadrature order to compute an integral
  //over x * boundary(x)
  int quadratureOrderBoundary () const
  {
    return 10;
  }

  //returns the recommended quadrature order to compute an integral
  //over x * interfaceBoundary(x)
  int quadratureOrderInterfaceBoundary () const
  {
    return 10;
  }

  //returns the recommended quadrature order to compute an integral
  //over x * qInterface(x)
  int quadratureOrder_qInterface () const
  {
    return 10;
  }

private:
  const Scalar d_; //aperture
};

#endif
