#ifndef __DUNE_MMDG_MMDGPROBLEM6_HH__
#define __DUNE_MMDG_MMDGPROBLEM6_HH__

#include <cmath>
#include <dune/mmdg/problems/coupleddgproblem.hh>

//a coupled dg problem
template<class Coordinate, class Scalar = double>
class MMDGProblem6 : public CoupledDGProblem<Coordinate, Scalar>
{
public:
  static constexpr int dim = Coordinate::dimension;

  //constructor
  MMDGProblem6 (const Scalar d0) : d0_(d0), xi_(1.0) {}

  //constructor
  MMDGProblem6 (const Scalar d0, const Scalar xi) : d0_(d0), xi_(xi) {}

  //the exact bulk solution at position pos
  Scalar exactSolution (const Coordinate& pos) const
  {
    Scalar xArg = (pos[0] < 0.5) ? sqrt(2.0) * (2.0 * pos[0] - 1.0)
      : sqrt(2.0) * (2.0 * pos[0] + 1.0);

    return std::cos(4.0 * pos[1]) * std::cosh(xArg);
  }

  //the exact solution on the interface at position pos
  Scalar exactInterfaceSolution (const Coordinate& pos) const
  {
    constexpr Scalar coshsqrt2 = std::cosh(sqrt(2.0));
    constexpr Scalar sinhsqrt2 = std::sinh(sqrt(2.0));

    return std::cos(4.0 * pos[1]) * (coshsqrt2 * coshsqrt2
      - (2 * xi_ - 1) * sinhsqrt2 * sinhsqrt2);
  }

  //indicates whether an exact solution is implemented for the problem
  bool hasExactSolution () const
  {
    return true;
  };

  //source term at position pos
  Scalar q (const Coordinate& pos) const
  {
    Scalar xArg = (pos[0] < 0.5) ? sqrt(2.0) * (2.0 * pos[0] - 1.0)
      : sqrt(2.0) * (2.0 * pos[0] + 1.0);

    return 8.0 * std::cos(4.0 * pos[1]) * std::cosh(xArg);
  }

  //interface source term at position pos
  Scalar qInterface (const Coordinate& pos) const
  {
    const Scalar cos4y = std::cos(4.0 * pos[1]);
    constexpr Scalar sqrt2 = 2.0 * sqrt(2.0);
    constexpr Scalar coshsqrt2 = std::cosh(sqrt(2.0));
    constexpr Scalar sinhsqrt2 = std::sinh(sqrt(2.0));
    const Scalar hyperbolicFactor =
      (coshsqrt2 * coshsqrt2 - (2 * xi_ - 1) * sinhsqrt2 * sinhsqrt2);

    return -16.0 * d0_ * d0_ * std::exp(-8.0 * pos[1]) * hyperbolicFactor * ( 
      cos4y + 3.0 * std::sin(4.0 * pos[1]) ) - sqrt2 * std::sinh(sqrt2) * cos4y;
  }

  //aperture d of the fracture at position pos
  Scalar aperture (const Coordinate& pos) const
  {
    return d0_ * std::exp(-4.0 * pos[1]);
  }

  //tangential gradient of the aperture d of the fracture at position pos
  Coordinate gradAperture (const Coordinate& pos) const
  {
    Coordinate gradD(0.0);
    gradD[1] = -4.0 * d0_ * std::exp(-4.0 * pos[1]);
    return gradD;
  }

  //permeability per aperture of the fracture in normal direction
  //at position pos
  Scalar Kperp (const Coordinate& pos) const
  {
    return sqrt(2.0) / std::tanh(sqrt(2.0));
  }

  //returns the recommended quadrature order to compute an integral
  //over x * boundary(x)
  int quadratureOrderBoundary () const
  {
    return 10;
  }

  //returns the recommended quadrature order to compute an integral
  //over  x * d^2 * interfaceBoundary(x)
  //and   x * d * interfaceBoundary(x) * (Kpar * grad(d))
  //and   d^2 * interfaceBoundary(x) * Kpar
  int quadratureOrderInterfaceBoundary () const
  {
    return 10;
  }

  //returns the recommended quadrature order to compute an integral
  //over x * qInterface(x)
  int quadratureOrder_qInterface () const
  {
    return 10;
  }

  //returns the recommended quadrature order to compute an integral
  //over x * q(x)
  int quadratureOrder_q () const
  {
    return 10;
  }

  //returns the recommended quadrature order to compute integrals
  //over  x^2 * (Kpar * grad(d) * grad(d))
  //and   x^2 * d^2
  //and   x^2 * d * (Kpar * grad(d))
  //and   x * d^2 * Kpar
  int quadratureOrder_Kparallel () const
  {
    return 10;
  }

private:
  Scalar d0_; //prefactor of the aperture
  Scalar xi_; //coupling constant

};

#endif
